//------------------------------------------------------------------
#property copyright "mladen"
#property link      "mladenfx@gmail.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 9
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_color4  Gold
#property indicator_color5  DimGray
#property indicator_color6  Red
#property indicator_color7  LimeGreen
#property indicator_color8  LimeGreen
#property indicator_color9  Orange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_style4  STYLE_DOT
#property indicator_style5  STYLE_DASH
#property indicator_minimum 0
#property indicator_maximum 100

//
//
//
//
//

enum enRsiTypes
{
   rsi_rsi,  // Regular RSI
   rsi_wil,  // Wilders RSI
   rsi_rsx,  // RSX
   rsi_cut   // Cuttlers RSI
};

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased   // Heiken ashi trend biased price
};
enum enTimeFrames
{
   tf_cu  = 0,              // Current time frame
   tf_m1  = PERIOD_M1,      // 1 minute
   tf_m5  = PERIOD_M5,      // 5 minutes
   tf_m15 = PERIOD_M15,     // 15 minutes
   tf_m30 = PERIOD_M30,     // 30 minutes
   tf_h1  = PERIOD_H1,      // 1 hour
   tf_h4  = PERIOD_H4,      // 4 hours
   tf_d1  = PERIOD_D1,      // Daily
   tf_w1  = PERIOD_W1,      // Weekly
   tf_mb1 = PERIOD_MN1      // Monthly
};
enum enApplyTo
{
   ap_rsi,  // Apply to RSI
   ap_mar   // Apply to signal line 
};

extern enTimeFrames       TimeFrame               = tf_cu;     // Time frame
extern int                StochasticLength        = 14;
extern int                SmoothEMA               = 9;
extern int                SignalEMA               = 5;
extern enRsiTypes         RsiType                 = rsi_rsx;
extern int                RsiPeriod               = 3;
extern enPrices           RsiPrice                = pr_haclose;
extern enPrices           VolatilityPrice         = pr_haclose;
extern int                VolatilityPeriod        = 5;
extern int                VolatilitySmooth        = 10;
extern bool               AdjustRsxPeriod         = true;
extern bool               AdjustStochasticPeriod  = true;
extern bool               ChangeOnDirectionChange = true;
extern int                TmaHalfLength           = 12;
extern enApplyTo          TmaApplyTo              = ap_rsi; 
extern double             BandsDeviations         = 3;
extern bool               drawDivergences           = true;
extern string             drawLinesIdentificator    = "Dssdiverge1";
extern double             DivergencearrowsUpperGap  = 0.1;
extern double             DivergencearrowsLowerGap  = 0.1;
extern bool               ShowClassicalDivergence   = true;
extern bool               ShowHiddenDivergence      = true;
extern bool               drawIndicatorTrendLines   = false;
extern bool               drawPriceTrendLines       = false;
extern color              divergenceBullishColor    = Lime;
extern color              divergenceBearishColor    = Magenta;
extern bool               divergenceAlert           = true;
extern bool               divergenceAlertsMessage   = true;
extern bool               divergenceAlertsSound     = true;
extern bool               divergenceAlertsEmail     = false;
extern bool               divergenceAlertsNotify    = false;
extern string             divergenceAlertsSoundName = "alert1.wav";
extern bool               ShowArrows              = false;
extern string             arrowsIdentifier        = "dssrsx Arrows1";
extern double             arrowsUpperGap          = 1.0;
extern double             arrowsLowerGap          = 1.0;
extern color              arrowsUpColor           = LimeGreen;
extern color              arrowsDnColor           = Red;
extern int                arrowsUpCode            = 241;
extern int                arrowsDnCode            = 242;
extern bool               alertsOn                = false;
extern bool               alertsOnCurrent         = true;
extern bool               alertsMessage           = true;
extern bool               alertsSound             = false;
extern bool               alertsEmail             = false;
extern bool               alertsNotification      = false; 
extern bool               Interpolate             = true;
extern int                DivergearrowSize        = 2.0;

//
//
//
//
//

double dssBuffer[];
double ddaBuffer[];
double ddbBuffer[];
double sigBuffer[];
double tmBuffer[];
double upBuffer[];
double dnBuffer[];
double bullishDivergence[];
double bearishDivergence[];
double trend[];
double pr[];
double tsum[];
double wrkBuffer[][15];
string indicatorFileName;
bool   returnBars;
string indicatorName;
string labelNames;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(12);
   SetIndexBuffer(0,dssBuffer); SetIndexDrawBegin(0,TmaHalfLength+1);
   SetIndexBuffer(1,ddaBuffer); SetIndexDrawBegin(1,TmaHalfLength+1);
   SetIndexBuffer(2,ddbBuffer); SetIndexDrawBegin(2,TmaHalfLength+1);
   SetIndexBuffer(3,sigBuffer); SetIndexDrawBegin(3,TmaHalfLength+1);
   SetIndexBuffer(4,tmBuffer);  SetIndexDrawBegin(4,TmaHalfLength+1);
   SetIndexBuffer(5,upBuffer);  SetIndexDrawBegin(5,TmaHalfLength+1);
   SetIndexBuffer(6,dnBuffer);  SetIndexDrawBegin(6,TmaHalfLength+1);
   SetIndexBuffer(7,bullishDivergence); SetIndexStyle(7,DRAW_ARROW,0,DivergearrowSize); SetIndexArrow(7,233); SetIndexDrawBegin(7,TmaHalfLength+1);
   SetIndexBuffer(8,bearishDivergence); SetIndexStyle(8,DRAW_ARROW,0,DivergearrowSize); SetIndexArrow(8,234); SetIndexDrawBegin(8,TmaHalfLength+1); 
   SetIndexBuffer(9,trend);
   SetIndexBuffer(10,pr);
   SetIndexBuffer(11,tsum);
   
   //
   //
   //
   //
   //
   
   TmaHalfLength     = MathMax(TmaHalfLength,1);
   StochasticLength  = MathMax(1,StochasticLength);
   SignalEMA         = MathMax(0,SignalEMA);
   SmoothEMA         = MathMax(0,SmoothEMA);
   if (SignalEMA<1) ChangeOnDirectionChange=true;
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame==-99; 
   TimeFrame         = MathMax(TimeFrame,_Period);
   labelNames        = "VA DSS RSI_DivergenceLine "+drawLinesIdentificator+":";
   indicatorName     = timeFrameToString(TimeFrame)+" VA DSS of "+getRsiName((int)RsiType)+" + tma abands ("+StochasticLength+","+SmoothEMA+","+SignalEMA+","+VolatilityPeriod+","+VolatilitySmooth+")";
   IndicatorShortName(indicatorName);
   return(0);
}

int deinit() 
{  
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   int length=StringLen(labelNames);
   for(i=ObjectsTotal()-1; i>=0; i--)
   {
      string name = ObjectName(i);
      if(StringSubstr(name,0,length) == labelNames)  ObjectDelete(name);   
   }
   return(0); 
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

#define wuBuffer 13
#define wdBuffer 14

int start()
{
   double FullLength = 2.0*TmaHalfLength+1.0;
   int v,r,j,k,n,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit = MathMin(Bars-1,Bars-counted_bars+TmaHalfLength);
           if (returnBars) { dssBuffer[0] = limit+1; return(0); }
           if (TimeFrame!=Period())
           {
               limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-98,0,0)*TimeFrame/Period()));
               if (trend[limit]==-1) CleanPoint(limit,ddaBuffer,ddbBuffer);
               for (int i=limit; i>=0; i--)
               {
                   int y = iBarShift(NULL,TimeFrame,Time[i]);               
                      dssBuffer[i] = iCustom(NULL,TimeFrame,indicatorFileName,-98,StochasticLength,SmoothEMA,SignalEMA,RsiType,RsiPeriod,RsiPrice,VolatilityPrice,VolatilityPeriod,VolatilitySmooth,AdjustRsxPeriod,AdjustStochasticPeriod,ChangeOnDirectionChange,TmaHalfLength,TmaApplyTo,BandsDeviations,
                                     drawDivergences,drawLinesIdentificator,DivergencearrowsUpperGap,DivergencearrowsLowerGap,ShowClassicalDivergence,ShowHiddenDivergence,drawIndicatorTrendLines,drawPriceTrendLines,divergenceBullishColor,divergenceBearishColor,divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsEmail,divergenceAlertsNotify,divergenceAlertsSoundName,
                                     ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotification,0,y);
                      sigBuffer[i] = iCustom(NULL,TimeFrame,indicatorFileName,-98,StochasticLength,SmoothEMA,SignalEMA,RsiType,RsiPeriod,RsiPrice,VolatilityPrice,VolatilityPeriod,VolatilitySmooth,AdjustRsxPeriod,AdjustStochasticPeriod,ChangeOnDirectionChange,TmaHalfLength,TmaApplyTo,BandsDeviations,
                                     drawDivergences,drawLinesIdentificator,DivergencearrowsUpperGap,DivergencearrowsLowerGap,ShowClassicalDivergence,ShowHiddenDivergence,drawIndicatorTrendLines,drawPriceTrendLines,divergenceBullishColor,divergenceBearishColor,divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsEmail,divergenceAlertsNotify,divergenceAlertsSoundName,
                                     ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotification,3,y);
                      tmBuffer[i]  = iCustom(NULL,TimeFrame,indicatorFileName,-98,StochasticLength,SmoothEMA,SignalEMA,RsiType,RsiPeriod,RsiPrice,VolatilityPrice,VolatilityPeriod,VolatilitySmooth,AdjustRsxPeriod,AdjustStochasticPeriod,ChangeOnDirectionChange,TmaHalfLength,TmaApplyTo,BandsDeviations,
                                     drawDivergences,drawLinesIdentificator,DivergencearrowsUpperGap,DivergencearrowsLowerGap,ShowClassicalDivergence,ShowHiddenDivergence,drawIndicatorTrendLines,drawPriceTrendLines,divergenceBullishColor,divergenceBearishColor,divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsEmail,divergenceAlertsNotify,divergenceAlertsSoundName,
                                     ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotification,4,y);
                      upBuffer[i]  = iCustom(NULL,TimeFrame,indicatorFileName,-98,StochasticLength,SmoothEMA,SignalEMA,RsiType,RsiPeriod,RsiPrice,VolatilityPrice,VolatilityPeriod,VolatilitySmooth,AdjustRsxPeriod,AdjustStochasticPeriod,ChangeOnDirectionChange,TmaHalfLength,TmaApplyTo,BandsDeviations,
                                     drawDivergences,drawLinesIdentificator,DivergencearrowsUpperGap,DivergencearrowsLowerGap,ShowClassicalDivergence,ShowHiddenDivergence,drawIndicatorTrendLines,drawPriceTrendLines,divergenceBullishColor,divergenceBearishColor,divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsEmail,divergenceAlertsNotify,divergenceAlertsSoundName,
                                     ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotification,5,y);
                      dnBuffer[i]  = iCustom(NULL,TimeFrame,indicatorFileName,-98,StochasticLength,SmoothEMA,SignalEMA,RsiType,RsiPeriod,RsiPrice,VolatilityPrice,VolatilityPeriod,VolatilitySmooth,AdjustRsxPeriod,AdjustStochasticPeriod,ChangeOnDirectionChange,TmaHalfLength,TmaApplyTo,BandsDeviations,
                                     drawDivergences,drawLinesIdentificator,DivergencearrowsUpperGap,DivergencearrowsLowerGap,ShowClassicalDivergence,ShowHiddenDivergence,drawIndicatorTrendLines,drawPriceTrendLines,divergenceBullishColor,divergenceBearishColor,divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsEmail,divergenceAlertsNotify,divergenceAlertsSoundName,
                                     ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotification,6,y);   
                      trend[i]     = iCustom(NULL,TimeFrame,indicatorFileName,-98,StochasticLength,SmoothEMA,SignalEMA,RsiType,RsiPeriod,RsiPrice,VolatilityPrice,VolatilityPeriod,VolatilitySmooth,AdjustRsxPeriod,AdjustStochasticPeriod,ChangeOnDirectionChange,TmaHalfLength,TmaApplyTo,BandsDeviations,
                                     drawDivergences,drawLinesIdentificator,DivergencearrowsUpperGap,DivergencearrowsLowerGap,ShowClassicalDivergence,ShowHiddenDivergence,drawIndicatorTrendLines,drawPriceTrendLines,divergenceBullishColor,divergenceBearishColor,divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsEmail,divergenceAlertsNotify,divergenceAlertsSoundName,
                                     ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotification,9,y);                           
                      ddaBuffer[i] = EMPTY_VALUE;   
                      ddbBuffer[i] = EMPTY_VALUE;
                      bullishDivergence[i] = EMPTY_VALUE;
                      bearishDivergence[i] = EMPTY_VALUE;
                   int firstBar = iBarShift(NULL,0,iTime(NULL,TimeFrame,y));
                   if (i==firstBar)
                   {
                      bullishDivergence[i] = iCustom(NULL,TimeFrame,indicatorFileName,-98,StochasticLength,SmoothEMA,SignalEMA,RsiType,RsiPeriod,RsiPrice,VolatilityPrice,VolatilityPeriod,VolatilitySmooth,AdjustRsxPeriod,AdjustStochasticPeriod,ChangeOnDirectionChange,TmaHalfLength,TmaApplyTo,BandsDeviations,
                                              drawDivergences,drawLinesIdentificator,DivergencearrowsUpperGap,DivergencearrowsLowerGap,ShowClassicalDivergence,ShowHiddenDivergence,drawIndicatorTrendLines,drawPriceTrendLines,divergenceBullishColor,divergenceBearishColor,divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsEmail,divergenceAlertsNotify,divergenceAlertsSoundName,
                                              ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotification,7,y);
                      bearishDivergence[i] = iCustom(NULL,TimeFrame,indicatorFileName,-98,StochasticLength,SmoothEMA,SignalEMA,RsiType,RsiPeriod,RsiPrice,VolatilityPrice,VolatilityPeriod,VolatilitySmooth,AdjustRsxPeriod,AdjustStochasticPeriod,ChangeOnDirectionChange,TmaHalfLength,TmaApplyTo,BandsDeviations,
                                              drawDivergences,drawLinesIdentificator,DivergencearrowsUpperGap,DivergencearrowsLowerGap,ShowClassicalDivergence,ShowHiddenDivergence,drawIndicatorTrendLines,drawPriceTrendLines,divergenceBullishColor,divergenceBearishColor,divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsEmail,divergenceAlertsNotify,divergenceAlertsSoundName,
                                              ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotification,8,y);
                   }
                      
                      //
                      //
                      //
                      //
                      //
      
                      if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

                      //
                      //
                      //
                      //
                      //

                      datetime time = iTime(NULL,TimeFrame,y);
                         for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
                         for(k = 1; k < n; k++)
                         {
                            dssBuffer[i+k] = dssBuffer[i] + (dssBuffer[i+n] - dssBuffer[i])* k/n;
                            sigBuffer[i+k] = sigBuffer[i] + (sigBuffer[i+n] - sigBuffer[i])* k/n;
                            tmBuffer[i+k]  = tmBuffer[i]  + (tmBuffer[i+n]  - tmBuffer[i]) * k/n;
                            upBuffer[i+k]  = upBuffer[i]  + (upBuffer[i+n]  - upBuffer[i]) * k/n;
                            dnBuffer[i+k]  = dnBuffer[i]  + (dnBuffer[i+n]  - dnBuffer[i]) * k/n;
                         }               
       }
       for (i=limit;i>=0;i--) if (trend[i]==-1) PlotPoint(i,ddaBuffer,ddbBuffer,dssBuffer);
   return(0);
   }

   //
   //
   //
   //
   //
   
   if (ArrayRange(wrkBuffer,0) != Bars) ArrayResize(wrkBuffer,Bars);
   double alpha = 2.0 / (1.0+SignalEMA);
   for (i=limit; i>=0; i--)
   {
      pr[i]   =  getPrice(VolatilityPrice,Open,Close,High,Low,i);
      tsum[i] =  0;            for (v = 0; v<VolatilityPeriod; v++) tsum[i] += MathAbs(pr[i+v]-pr[i+v+1]);
      double avg  =  tsum[i];  for (v = 1; v<VolatilitySmooth; v++) avg     += tsum[i+v];
             avg /= VolatilitySmooth;
         
                //
                //
                //
                //
                //
                
                if (AdjustRsxPeriod && avg!=0 && tsum[i]!=0)
                     double rsiPer = MathMin(MathMax(RsiPeriod/(tsum[i]/avg),RsiPeriod/2.0),RsiPeriod*5.0);
                else        rsiPer = RsiPeriod;
                if (AdjustStochasticPeriod && avg!=0 && tsum[i]!=0)
                     double stoPer = MathMin(MathMax(StochasticLength/(tsum[i]/avg),StochasticLength/2.0),StochasticLength*5.0);
                else        stoPer = StochasticLength;
                
     
                double rsx   = iRsi(getPrice(RsiPrice,Open,Close,High,Low,i),rsiPer,RsiType,i);
                dssBuffer[i] = iDss(rsx,rsx,rsx,stoPer,SmoothEMA,i);
                if (drawDivergences)
                {
                  CatchBullishDivergence(i);
                  CatchBearishDivergence(i);
                }            
           }
           for (i=limit; i>=0; i--) sigBuffer[i] = sigBuffer[i+1]+alpha*(dssBuffer[i]-sigBuffer[i+1]);
           
           //
           //
           //
           //
           //
                
           if (trend[limit]==-1) CleanPoint(limit,ddaBuffer,ddbBuffer);    
           for(i=limit, r=Bars-i-1; i>=0; i--, r++)
           {
              ddaBuffer[i] = EMPTY_VALUE;   
              ddbBuffer[i] = EMPTY_VALUE;   
              trend[i]     = trend[i+1];

              if (ChangeOnDirectionChange)
              {
                 if (dssBuffer[i]>dssBuffer[i+1]) trend[i] =  1;
                 if (dssBuffer[i]<dssBuffer[i+1]) trend[i] = -1;
              }
              else
              {
                 if (dssBuffer[i]>sigBuffer[i+1]) trend[i] =  1;
                 if (dssBuffer[i]<sigBuffer[i+1]) trend[i] = -1;
              }               
              if (trend[i]==-1) PlotPoint(i,ddaBuffer,ddbBuffer,dssBuffer);  
               
              if (TmaApplyTo==0)
              double price = dssBuffer[i];
              else   price = sigBuffer[i];

         //
         //
         //
         //
         //
      
         double sum  = (TmaHalfLength+1)*price;
         double sumw = (TmaHalfLength+1);
         for(j=1, k=TmaHalfLength; j<=TmaHalfLength; j++, k--)
         {
            if (TmaApplyTo==0)               
                  sum  += k*dssBuffer[i+j];
            else  sum  += k*sigBuffer[i+j];
            sumw += k;

            if (j<=i)
            {
               if (TmaApplyTo==0)               
                     sum  += k*dssBuffer[i-j];
               else  sum  += k*sigBuffer[i-j];
               sumw += k;
            }
         }
         tmBuffer[i] = sum/sumw;
   
      //
      //
      //
      //
      //
            
         double diff = price-tmBuffer[i];
            if (i> (Bars-TmaHalfLength-2)) continue;
            if (i==(Bars-TmaHalfLength-2))
            {
               upBuffer[i] = tmBuffer[i];
               dnBuffer[i] = tmBuffer[i];
               if (diff>=0)
                  {
                     wrkBuffer[r][wuBuffer] = MathPow(diff,2);
                     wrkBuffer[r][wdBuffer] = 0;
                  }
               else
                  {               
                     wrkBuffer[r][wdBuffer] = MathPow(diff,2);
                     wrkBuffer[r][wuBuffer] = 0;
                  }                  
               continue;
            }
      
         //
         //
         //
         //
         //
         
         wrkBuffer[r][wuBuffer] = wrkBuffer[r-1][wuBuffer];
         wrkBuffer[r][wdBuffer] = wrkBuffer[r-1][wdBuffer];
            if(diff>0)
            {
               wrkBuffer[r][wuBuffer] = (wrkBuffer[r-1][wuBuffer]*(FullLength-1)+MathPow(diff,2))/FullLength;
               wrkBuffer[r][wdBuffer] =  wrkBuffer[r-1][wdBuffer]*(FullLength-1)/FullLength;
            }
            if(diff<0)
            {
               wrkBuffer[r][wdBuffer] = (wrkBuffer[r-1][wdBuffer]*(FullLength-1)+MathPow(diff,2))/FullLength;
               wrkBuffer[r][wuBuffer] =  wrkBuffer[r-1][wuBuffer]*(FullLength-1)/FullLength;
            }
         upBuffer[i] = tmBuffer[i] + BandsDeviations*MathSqrt(wrkBuffer[r][wuBuffer]);
         dnBuffer[i] = tmBuffer[i] - BandsDeviations*MathSqrt(wrkBuffer[r][wdBuffer]);
               
        //
        //
        //
        //
        //
               
        if (ShowArrows)
        {
          ObjectDelete(arrowsIdentifier+":"+Time[i]);
          if (trend[i] != trend[i+1])
          {
            if (trend[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
            if (trend[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
          }
       }
   }
   manageAlerts();
   return(0);           
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workDss[][5];
#define _plow  0
#define _phigh 1
#define _st1   2
#define _ss1   3
#define _dss   4

double iDss(double pClose, double pHigh, double pLow, int period, int smoothPeriod, int i, int instanceNo=0)
{
   if (ArrayRange(workDss,0)!=Bars) ArrayResize(workDss,Bars); i = Bars-i-1; instanceNo*=5;

   //
   //
   //
   //
   //
   
         double alpha = 2.0 / (1.0+smoothPeriod);
            workDss[i][instanceNo+_phigh] = pHigh;
            workDss[i][instanceNo+_plow]  = pLow;
               double min   = workDss[i][instanceNo+_plow];
               double max   = workDss[i][instanceNo+_phigh];

      
         for (int k=1; k<period && (i-k)>=0; k++)
         {
            min = MathMin(min,workDss[i-k][instanceNo+_plow] );
            max = MathMax(max,workDss[i-k][instanceNo+_phigh]);
         }
         workDss[i][instanceNo+_st1] = 0; if (min!=max) workDss[i][instanceNo+_st1] = 100*(pClose-min)/(max-min);
         
         //
         //
         //
         //
         //
         
         if (i==0)
         {
               workDss[i][instanceNo+_ss1] = workDss[i][instanceNo+_st1];
               workDss[i][instanceNo+_dss] = workDss[i][instanceNo+_st1];
         }            
         else
         {
            workDss[i][instanceNo+_ss1] = workDss[i-1][instanceNo+_ss1]+alpha*(workDss[i][instanceNo+_st1]-workDss[i-1][instanceNo+_ss1]);
               min = workDss[i][instanceNo+_ss1];
               max = workDss[i][instanceNo+_ss1];
               for (k=1; k<period && (i-k)>=0; k++)
               {
                  min = MathMin(min,workDss[i-k][instanceNo+_ss1]);
                  max = MathMax(max,workDss[i-k][instanceNo+_ss1]);
               }
               double stoch = 0; if (min!=max) stoch = 100*(workDss[i][instanceNo+_ss1]-min)/(max-min);
               workDss[i][instanceNo+_dss] = workDss[i-1][instanceNo+_dss]+alpha*(stoch-workDss[i-1][instanceNo+_dss]);
         }            
            
      //
      //
      //
      //
      //
            
   return(workDss[i][instanceNo+_dss]);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//
//

string rsiMethodNames[] = {"rsi","Wilders rsi","rsx","Cuttler RSI"};
string getRsiName(int method)
{
   int max = ArraySize(rsiMethodNames)-1;
      method=MathMax(MathMin(method,max),0); return(rsiMethodNames[method]);
}

//
//
//
//
//

double workRsi[][13];
#define _price  0
#define _change 1
#define _changa 2

double iRsi(double price, double period, int rsiMode, int i, int instanceNo=0)
{
   if (ArrayRange(workRsi,0)!=Bars) ArrayResize(workRsi,Bars);
      int z = instanceNo*13; 
      int r = Bars-i-1;
   
   //
   //
   //
   //
   //
   
   workRsi[r][z+_price] = price;
   switch (rsiMode)
   {
      case 0:
         double alpha = 1.0/period; 
         if (r<period)
            {
               int k; double sum = 0; for (k=0; k<period && (r-k-1)>=0; k++) sum += MathAbs(workRsi[r-k][z+_price]-workRsi[r-k-1][z+_price]);
                  workRsi[r][z+_change] = (workRsi[r][z+_price]-workRsi[0][z+_price])/MathMax(k,1);
                  workRsi[r][z+_changa] =                                         sum/MathMax(k,1);
            }
         else
            {
               double change = workRsi[r][z+_price]-workRsi[r-1][z+_price];
                               workRsi[r][z+_change] = workRsi[r-1][z+_change] + alpha*(        change  - workRsi[r-1][z+_change]);
                               workRsi[r][z+_changa] = workRsi[r-1][z+_changa] + alpha*(MathAbs(change) - workRsi[r-1][z+_changa]);
            }
         if (workRsi[r][z+_changa] != 0)
               return(50.0*(workRsi[r][z+_change]/workRsi[r][z+_changa]+1));
         else  return(50.0);
         
      //
      //
      //
      //
      //
      
      case 1 :
         workRsi[r][z+1] = iSmma(0.5*(MathAbs(workRsi[r][z+_price]-workRsi[r-1][z+_price])+(workRsi[r][z+_price]-workRsi[r-1][z+_price])),0.5*(period-1),Bars-i-1,instanceNo*2+0);
         workRsi[r][z+2] = iSmma(0.5*(MathAbs(workRsi[r][z+_price]-workRsi[r-1][z+_price])-(workRsi[r][z+_price]-workRsi[r-1][z+_price])),0.5*(period-1),Bars-i-1,instanceNo*2+1);
         if((workRsi[r][z+1] + workRsi[r][z+2]) != 0) 
               return(100.0 * workRsi[r][z+1]/(workRsi[r][z+1] + workRsi[r][z+2]));
         else  return(50);

      //
      //
      //
      //
      //

      case 2 :     
         double Kg = (3.0)/(2.0+period), Hg = 1.0-Kg;
         if (r<period) { for (k=1; k<13; k++) workRsi[r][k+z] = 0; return(50); }  

         //
         //
         //
         //
         //
      
         double mom = workRsi[r][_price+z]-workRsi[r-1][_price+z];
         double moa = MathAbs(mom);
         for (k=0; k<3; k++)
         {
            int kk = k*2;
            workRsi[r][z+kk+1] = Kg*mom                + Hg*workRsi[r-1][z+kk+1];
            workRsi[r][z+kk+2] = Kg*workRsi[r][z+kk+1] + Hg*workRsi[r-1][z+kk+2]; mom = 1.5*workRsi[r][z+kk+1] - 0.5 * workRsi[r][z+kk+2];
            workRsi[r][z+kk+7] = Kg*moa                + Hg*workRsi[r-1][z+kk+7];
            workRsi[r][z+kk+8] = Kg*workRsi[r][z+kk+7] + Hg*workRsi[r-1][z+kk+8]; moa = 1.5*workRsi[r][z+kk+7] - 0.5 * workRsi[r][z+kk+8];
         }
         if (moa != 0)
              return(MathMax(MathMin((mom/moa+1.0)*50.0,100.00),0.00)); 
         else return(50);
            
      //
      //
      //
      //
      //
      
      case 3 :
         double sump = 0;
         double sumn = 0;
         for (k=0; k<period; k++)
         {
            double diff = workRsi[r-k][z+_price]-workRsi[r-k-1][z+_price];
               if (diff > 0) sump += diff;
               if (diff < 0) sumn -= diff;
         }
         if (sumn > 0)
               return(100.0-100.0/(1.0+sump/sumn));
         else  return(50);
   } 
   return(0);
}

//
//
//
//
//
//

double workSmma[][2];
double iSmma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workSmma,0)!= Bars) ArrayResize(workSmma,Bars);

   if (r<period)
         workSmma[r][instanceNo] = price;
   else  workSmma[r][instanceNo] = workSmma[r-1][instanceNo]+(price-workSmma[r-1][instanceNo])/period;
   return(workSmma[r][instanceNo]);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1;if (alertsOnCurrent) whichBar = 0;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"changed trend to up");
         if (trend[whichBar] == -1) doAlert(whichBar,"changed trend to down");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(timeFrameToString(Period())+" "+Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," VA DSS of RSX ",doWhat);
          if (alertsMessage)      Alert(message);
          if (alertsEmail)        SendMail(StringConcatenate(Symbol()," VA DSS of Rsi "),message);
          if (alertsNotification) SendNotification(StringConcatenate(Symbol()," VA DSS of Rsi ",message));
          if (alertsSound)        PlaySound("alert2.wav");
   }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//
//

double workHa[][4];
double getPrice(int tprice, const double& open[], const double& close[], const double& high[], const double& low[], int i, int instanceNo=0)
{
  if (tprice>=pr_haclose && tprice<=pr_hatbiased)
   {
      if (ArrayRange(workHa,0)!= Bars) ArrayResize(workHa,Bars);
         int r = Bars-i-1;
         
         //
         //
         //
         //
         //
         
         double haOpen;
         if (r>0)
                haOpen  = (workHa[r-1][instanceNo+2] + workHa[r-1][instanceNo+3])/2.0;
         else   haOpen  = (open[i]+close[i])/2;
         double haClose = (open[i] + high[i] + low[i] + close[i]) / 4.0;
         double haHigh  = MathMax(high[i], MathMax(haOpen,haClose));
         double haLow   = MathMin(low[i] , MathMin(haOpen,haClose));

         if(haOpen  <haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else                 { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                                workHa[r][instanceNo+2] = haOpen;
                                workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (tprice)
         {
            case pr_haclose:     return(haClose);
            case pr_haopen:      return(haOpen);
            case pr_hahigh:      return(haHigh);
            case pr_halow:       return(haLow);
            case pr_hamedian:    return((haHigh+haLow)/2.0);
            case pr_hamedianb:   return((haOpen+haClose)/2.0);
            case pr_hatypical:   return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:  return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:   return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (tprice)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
   }
   return(0);
}     

//
//
//
//
//

void CatchBullishDivergence(int shift)
{
   shift++;
         bullishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"l"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"l"+"os" + DoubleToStr(Time[shift],0));            
   if(!IsIndicatorLow(shift)) return;  
   int currentLow = shift;
   int lastLow    = GetIndicatorLastLow(shift+1);
   
   //
   //
   //
   //
   //
   
   if (dssBuffer[currentLow] > dssBuffer[lastLow] && Low[currentLow] < Low[lastLow])
   {
     if (ShowClassicalDivergence)
     {
        bullishDivergence[currentLow] = dssBuffer[currentLow] - DivergencearrowsLowerGap;
        if (drawPriceTrendLines)    DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],                divergenceBullishColor,STYLE_SOLID);
        if (drawIndicatorTrendLines)DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],dssBuffer[currentLow],dssBuffer[lastLow],divergenceBullishColor,STYLE_SOLID);
        if (divergenceAlert)        DisplayAlert("Classical bullish divergence",currentLow);  
     }
                        
   }
     
   //
   //
   //
   //
   //
        
   if (dssBuffer[currentLow] < dssBuffer[lastLow] && Low[currentLow] > Low[lastLow])
   {
     if (ShowHiddenDivergence)
     {
        bullishDivergence[currentLow] = dssBuffer[currentLow] - DivergencearrowsLowerGap;
        if (drawPriceTrendLines)     DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],                divergenceBullishColor, STYLE_DOT);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],dssBuffer[currentLow],dssBuffer[lastLow],divergenceBullishColor, STYLE_DOT);
        if (divergenceAlert)         DisplayAlert("Reverse bullish divergence",currentLow); 
     }
   }
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CatchBearishDivergence(int shift)
{
   shift++; 
         bearishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"h"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"h"+"os" + DoubleToStr(Time[shift],0));            
   if(!IsIndicatorPeak(shift)) return;
   int currentPeak = shift;
   int lastPeak    = GetIndicatorLastPeak(shift+1);

   //
   //
   //
   //
   //
      
   if (dssBuffer[currentPeak] < dssBuffer[lastPeak] && High[currentPeak]>High[lastPeak])
   {
     if (ShowClassicalDivergence)
     {
        bearishDivergence[currentPeak] = dssBuffer[currentPeak] + DivergencearrowsUpperGap;
        if (drawPriceTrendLines)     DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],              divergenceBearishColor,STYLE_SOLID);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],dssBuffer[currentPeak],dssBuffer[lastPeak],divergenceBearishColor,STYLE_SOLID);
        if (divergenceAlert)         DisplayAlert("Classical bearish divergence",currentPeak);
     } 
                        
   }
   
   //
   //
   //
   //
   //

   if (dssBuffer[currentPeak] > dssBuffer[lastPeak] && High[currentPeak] < High[lastPeak])
   {
     if (ShowHiddenDivergence)
     {
        bearishDivergence[currentPeak] = dssBuffer[currentPeak] + DivergencearrowsUpperGap;
        if (drawPriceTrendLines)     DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],              divergenceBearishColor, STYLE_DOT);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],dssBuffer[currentPeak],dssBuffer[lastPeak],divergenceBearishColor, STYLE_DOT);
        if (divergenceAlert)         DisplayAlert("Reverse bearish divergence",currentPeak);
     }
                         
   }   
}

//
//
//
//
//

bool IsIndicatorPeak(int shift)
{
   if(dssBuffer[shift] >= dssBuffer[shift+1] && dssBuffer[shift] > dssBuffer[shift+2] && dssBuffer[shift] > dssBuffer[shift-1])
       return(true);
   else 
       return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

bool IsIndicatorLow(int shift)
{
   if(dssBuffer[shift] <= dssBuffer[shift+1] && dssBuffer[shift] < dssBuffer[shift+2] && dssBuffer[shift] < dssBuffer[shift-1])
       return(true);
   else 
       return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastPeak(int shift)
{
    for(int i = shift+5; i < Bars; i++)
    {
       if(dssBuffer[i] >= dssBuffer[i+1] && dssBuffer[i] > dssBuffer[i+2] && dssBuffer[i] >= dssBuffer[i-1] && dssBuffer[i] > dssBuffer[i-2])
    return(i);
    }
return(-1);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastLow(int shift)
{
    for (int i = shift+5; i < Bars; i++)
    {
       if (dssBuffer[i] <= dssBuffer[i+1] && dssBuffer[i] < dssBuffer[i+2] && dssBuffer[i] <= dssBuffer[i-1] && dssBuffer[i] < dssBuffer[i-2])
    return(i);
    }
     
return(-1);
}

//
//
//
//
//

void DisplayAlert(string doWhat, int shift)
{
    string dmessage;
    static datetime lastAlertTime;
    if(shift <= 2 && Time[0] != lastAlertTime)
    {
      dmessage =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," VA DSS of Rsi ",doWhat);
          if (divergenceAlertsMessage) Alert(dmessage);
          if (divergenceAlertsNotify)  SendNotification(dmessage);
          if (divergenceAlertsEmail)   SendMail(StringConcatenate(Symbol()," VA DSS of Rsi "),dmessage);
          if (divergenceAlertsSound)   PlaySound(divergenceAlertsSoundName); 
          lastAlertTime = Time[0];
    }
}

//
//
//
//
//

void DrawPriceTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    string label = labelNames+first+"os"+DoubleToStr(t1,0);
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, 0, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

//
//
//
//
//

void DrawIndicatorTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    int indicatorWindow = WindowFind(indicatorName);
    if (indicatorWindow < 0) return;
    
    //
    //
    //
    //
    //
    
    string label = labelNames+first+DoubleToStr(t1,0);
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, indicatorWindow, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

